/* Exif-display Plugin - GSettings Keys Macros
 *
 * Copyright (C) 2011 The Free Software Foundation
 *
 * Author: Felix Riemann  <friemann@gnome.org>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

#ifndef __EOG_EXIF_DISPLAY_PLUGIN_SETTINGS_H__
#define __EOG_EXIF_DISPLAY_PLUGIN_SETTINGS_H__

#define EOG_EXIF_DISPLAY_CONF_SCHEMA_ID "org.gnome.eog.plugins.exif-display"

#define EOG_EXIF_DISPLAY_CONF_CHANNELS_HISTOGRAM	"channels-histogram"
#define EOG_EXIF_DISPLAY_CONF_RGB_HISTOGRAM		"rgb-histogram"
#define EOG_EXIF_DISPLAY_CONF_EXIF_IN_STATUSBAR		"exif-in-statusbar"

#endif /* __EOG_EXIF_DISPLAY_PLUGIN_SETTINGS_H__*/
